bl_info = {
    "name": "Show Object/Data Users in Viewport",
    "author": "1P2DD",
    "version": (1, 0),
    "blender": (4, 0, 0),
    "location": "View3D > Object > Viewport Display",
    "description": "Display the number of data and object users in the 3D viewport",
    "category": "3D View",
}

import bpy
import blf
from bpy_extras.view3d_utils import location_3d_to_region_2d

handle = None

def register():
    global handle
    # Define BoolProperties for displaying data and object users
    bpy.types.Object.show_data_users = bpy.props.BoolProperty(
        name="Data Users",
        description="Display the number of data users (mesh sharing) in the viewport",
        default=False
    )
    bpy.types.Object.show_object_users = bpy.props.BoolProperty(
        name="Object Users",
        description="Display the number of object users (linked across scenes) in the viewport",
        default=False
    )

    # Append the properties to the UI
    bpy.types.OBJECT_PT_display.append(draw_users_properties)

    # Add the draw handler for the viewport overlay
    if handle is None:
        handle = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, (), 'WINDOW', 'POST_PIXEL')

def unregister():
    global handle
    # Remove the custom properties
    del bpy.types.Object.show_data_users
    del bpy.types.Object.show_object_users

    # Remove the UI panel
    bpy.types.OBJECT_PT_display.remove(draw_users_properties)

    # Remove the draw handler
    if handle is not None:
        bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
        handle = None

def draw_users_properties(self, context):
    layout = self.layout
    obj = context.object
    if obj is None:
        return
    col = layout.column(align=True)

    # Check if the object has data
    has_data = hasattr(obj, 'data') and obj.data is not None

    # Create a row for "Data Users" and set its enabled state based on 'has_data'
    row = col.row(align=True)
    row.enabled = has_data
    row.prop(obj, "show_data_users", text="Data Users")

    # "Object Users" is always enabled
    col.prop(obj, "show_object_users", text="Object Users")

def count_object_users(obj):
    """
    Count how many scenes have this object.
    This is particularly useful for empties instancing collections linked from the Asset Browser.
    """
    if obj.type == 'EMPTY' and obj.instance_type == 'COLLECTION' and obj.instance_collection:
        # Count how many times the collection is instanced across all scenes
        collection = obj.instance_collection
        count = 0
        for scene in bpy.data.scenes:
            for other_obj in scene.collection.all_objects:
                if (other_obj.type == 'EMPTY' and 
                    other_obj.instance_type == 'COLLECTION' and 
                    other_obj.instance_collection == collection):
                    count += 1
        return count
    else:
        # For other object types, use obj.users
        return obj.users

def draw_callback_px():
    context = bpy.context

    # Ensure we're in a 3D View
    if not isinstance(context.space_data, bpy.types.SpaceView3D):
        return

    # Ensure overlays are enabled
    if not context.space_data.overlay.show_overlays:
        return

    font_id = 0  # Default font ID
    blf.size(font_id, 11)

    # Access the theme for consistent coloring
    theme = context.preferences.themes[0]
    view_3d_theme = theme.view_3d

    # Define theme colors
    active_color = (*view_3d_theme.object_active, 1.0)
    selected_color = (*view_3d_theme.object_selected, 1.0)
    unselected_color = (*view_3d_theme.wire, 1.0)
    edit_mode_color = (*view_3d_theme.wire_edit, 1.0)

    # X offset for text positioning
    x_offset = 10

    # Find the active 3D View region
    for area in context.screen.areas:
        if area.type == 'VIEW_3D':
            for region in area.regions:
                if region.type == 'WINDOW':
                    break
            break
    else:
        return  # Exit if no 3D View is found

    rv3d = area.spaces[0].region_3d

    # Iterate through visible objects
    for obj in context.visible_objects:
        try:
            # Check if either display property is enabled
            if obj.show_data_users or obj.show_object_users:
                # Get object's world coordinates
                obj_location = obj.matrix_world.translation

                # Convert 3D coordinates to 2D screen space
                coord_2d = location_3d_to_region_2d(region, rv3d, obj_location)

                if coord_2d:
                    texts = []

                    # Handle Data Users
                    if obj.show_data_users:
                        if hasattr(obj, 'data') and obj.data:
                            texts.append(f"Data users: {obj.data.users}")
                        else:
                            # Disable the "Data Users" checkbox if no data
                            obj.show_data_users = False
                            # Optionally, you can skip adding the text
                            # texts.append("Data users: N/A")

                    # Handle Object Users
                    if obj.show_object_users:
                        object_users = count_object_users(obj)
                        texts.append(f"Object users: {object_users}")

                    # Determine text color based on object state
                    if obj.mode == 'EDIT':
                        color = edit_mode_color
                    elif obj.select_get():
                        if obj == context.view_layer.objects.active:
                            color = active_color
                        else:
                            color = selected_color
                    else:
                        color = unselected_color

                    # Set the text color
                    blf.color(font_id, *color)

                    # Initialize Y offset for text placement
                    y_offset = coord_2d.y

                    # Adjust position if object name is displayed
                    if obj.show_name:
                        object_name = obj.name
                        blf.position(font_id, coord_2d.x + x_offset, coord_2d.y, 0)
                        name_width, name_height = blf.dimensions(font_id, object_name)
                        y_offset -= name_height + 2
                    else:
                        y_offset = coord_2d.y

                    # Draw each text line
                    for text in texts:
                        blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                        blf.draw(font_id, text)
                        _, text_height = blf.dimensions(font_id, text)
                        y_offset -= text_height + 2

        except Exception as e:
            print(f"Error processing object '{obj.name}': {e}")

if __name__ == "__main__":
    register()
